<!DOCTYPE html>
<html lang="en" class="text-gray-900 antialiased leading-tight font-sans">
  <head>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1" />
    <title>gdbgui - dashboard</title>
    <link rel="shortcut icon" href="{{ url_for('static', filename='favicon.ico') }}" />
  </head>

  <body class="w-screen h-screen bg-gray-300">
    <div id="dashboard">Loading dashboard...</div>
  </body>
  <script>
    window.gdbgui_sessions = {{gdbgui_sessions | tojson}}
    window.csrf_token = {{csrf_token | tojson}}
    window.default_command = {{default_command | tojson}}
  </script>
  <script type="text/javascript" src="static/js/dashboard.js?_={{version}}"></script>
</html>
